<!DOCTYPE html>
<html>

<head>
  <meta charset="utf-8" />
  <title>GramexNLG Template Generator</title>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <link rel="stylesheet" href="ui/bootstraptheme.css">
  <link rel="stylesheet" href="ui/font-awesome/css/font-awesome.min.css">
</head>

<script src="ui/jquery/dist/jquery.min.js"></script>
<script src="ui/bootstrap/dist/js/bootstrap.bundle.min.js"></script>
<script src="ui/lodash/lodash.min.js"></script>
<script src="ui/g1/dist/g1.min.js"></script>

<body>
  {% from gramex import variables %}
  {% set nlg_base = '.' %}
  <div class="container-fluid">
    {% include "./body.html" %}
  </div>
</body>

</html>
